#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>

#include <time.h>
#include <sys/time.h>

#include "mcp4725.h"

#define MAIN_C
#include "exitfail.h"

#define BASENAME(p) ((strrchr((p), '/') ? : ((p)-1))+1)

#define I2C_ADDR	0x60 /* Device Address(when ADDR pin: GND) 1100000=0x60 */
#define DEFAULT_STEP 0

static void usage(char *prog)
{
	printf("Usage: %s <-d|--device FILENAME> [-s|--step STEP]\n", BASENAME(prog));
}

static void parse_arg(int argc, char *argv[], char **device, int *step)
{
	int c;
	char *endptr;

	*device = NULL;
	for(;;) {
		int option_index = 0;
		static struct option long_options[] = {
			{"device",	required_argument,	NULL,	'd'},
			{"step",	required_argument,	NULL,	's'},
			{0,		0,			0,	0},
		};

		c = getopt_long(argc, argv, "d:s:",
				  long_options, &option_index);

		if(c == -1) break;
		
		switch(c) {
		case 'd':
			*device = optarg;
			break;
		case 's':
			errno = 0;
			*step = strtol(optarg, &endptr, 0);
			if(errno != 0 || optarg == endptr) 
				goto err;

			if(*step < MCP4725_STEP_MIN || MCP4725_STEP_MAX < *step)
				goto err;
			break;
		default:
			goto err;
		}
	}

	if(*device == NULL) goto err;
	return;

err:
	usage(argv[0]);
	exit(EXIT_FAILURE);
}

int c2toi_12bit(int c2){
	int ret;

	if(c2 > 0x800) ret = -(0x1000 - c2);
	else ret = c2;

	return ret;
}

int main(int argc, char *argv[])
{
	struct mcp4725 *sens;
	char *device;
	int step = DEFAULT_STEP;

	int ret;

	exitfail_init();

	parse_arg(argc, argv, &device, &step);

	sens = mcp4725_open(device, I2C_ADDR);
	if(sens == NULL) exitfail_errno("mcp4725_open");

	/**** MCP4725: Write Register: begin ****/
	ret = mcp4725_write(sens, 0x0, step);
	if(ret !=0) exitfail_errno("mcp4725_write");

	/**** MCP4725: Read Register(CTRL_REG1: 0x2A) for ACTIVE: end ****/

	ret = mcp4725_close(sens);

	if(ret != 0) exitfail_errno("mcp4725_close");

	return EXIT_SUCCESS;
}
