#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <ctype.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <linux/i2c-dev.h>

#include <stdio.h> 
#include <time.h> 

/**** sample: for I2C SMBUS: begin ****/
#include <linux/i2c.h>
/**** sample: for I2C SMBUS: end ****/

#include "mcp4725.h"

#define MCP4725_RETRY_MAX 3
#define MCP4725_REG_MIN 0x80
#define MCP4725_REG_MAX 0x8f

struct mcp4725 {
	int fd;
};

static ssize_t write_uninterruptible(int fd, const void *buf, size_t count, int retry)
{
	ssize_t ret;
	int i;

	for(i=0; i<retry; i++) {
		ret = write(fd, buf, count);
		if(ret < 0 && errno != EINTR) return -1;
		if((size_t)ret == count) return ret;
	}
	errno = ETIMEDOUT;
	return -1;
}

struct mcp4725 *mcp4725_open(const char *dev_path, const int addr)
{
	struct mcp4725 *sens;
	int error;
	int ret;

	if(dev_path == NULL) {
		errno = EINVAL;
		return NULL;
	}

	sens = calloc(1, sizeof(struct mcp4725));
	if(sens == NULL) return NULL;

	sens->fd = open(dev_path, O_RDWR);
	if(sens->fd < 0) {
		error = errno;
		goto err1;
	}

	ret = ioctl(sens->fd, I2C_SLAVE, addr);
	if(ret < 0) {
		error = errno;
		goto err2;
	}

	return sens;

err2:
	close(sens->fd);
err1:
	free(sens);
	errno = error;
	return NULL;
}

int mcp4725_write(struct mcp4725 *sens, uint8_t pd, int step)
{
//	int i;
	int ret;
	uint8_t buf[2];

	if(sens == NULL || /* pd < 0 || */ pd > 3 || step < 0 || step > 4095) {
		errno = EINVAL;
		return -1;
	}

	buf[0] = pd << 4 | step >> 8;
	buf[1] = step & 0xFF;

	ret = write_uninterruptible(sens->fd, buf, 2 + 1, MCP4725_RETRY_MAX);
	if(ret < 0) return -1;

	return 0;
}

int mcp4725_close(struct mcp4725 *sens)
{
	int fd;

	if(sens == NULL) {
		errno = EINVAL;
		return -1;
	}

	fd = sens->fd;
	free(sens);

	return close(fd);
}

